/***************************************************************************
                          kcountrypage.cpp  -  description
                             -------------------
    begin                : Tue May 22 2001
    copyright            : (C) 2001 by Ralf Nolden
    email                : nolden@kde.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qstringlist.h>
#include <qlabel.h>
#include <qmap.h>

#include <kapplication.h>
#include <ksimpleconfig.h>
#include <klocale.h>
#include <kdebug.h>
#include <kstandarddirs.h>
#include <dcopclient.h>
#include <kprocess.h>

#include "klanguagebutton.h"
#include "kfindlanguage.h"

#include "kcountrypage.h"

KCountryPage::KCountryPage(QWidget *parent, const char *name ) : KCountryPageDlg(parent,name) {

	px_introSidebar->setPixmap(locate("data", "kpersonalizer/pics/step1.png"));

	connect(cb_country, SIGNAL(activated(int)), SLOT(setLangForCountry(int)));
	connect(cb_language, SIGNAL(activated(int)), SLOT(setLanguageChanged()));

	// naturally, the language is not changed on startup
	b_savedLanguageChanged = false;
	b_startedLanguageChanged = false;

	// set appropriate KDE version (kapplication.h)
	txt_welcome->setText(i18n("<h3>Welcome to KDE %1!</h3>").arg(KDE_VERSION_STRING));

	flang = new KFindLanguage();

	// need this ones for decision over restarts of kp, kicker, etc
	s_oldlocale = KGlobal::locale()->language();

	// load the Menus and guess the selection
	loadCountryList(cb_country);
	fillLanguageMenu(cb_language);
	cb_language->setCurrentItem(flang->getBestLang());

	// Highlight the users's country
	for(int i = 0; i < cb_country->count(); i++) {
		if(cb_country->tag(i).contains(flang->getCountry())) {
			cb_country->setCurrentItem(i);
		}
	}

	setLanguageChanged();
}

KCountryPage::~KCountryPage(){
	delete flang;
}


void KCountryPage::loadCountryList(KLanguageButton *combo) {

	QString sub = QString::fromLatin1("l10n/");

	// clear the list
	combo->clear();

	QStringList regionlist = KGlobal::dirs()->findAllResources("locale", sub + "*.desktop");
	regionlist.sort();

	for ( QStringList::ConstIterator it = regionlist.begin(); it != regionlist.end(); ++it ) {
		QString tag = *it;
		int index;

		index = tag.findRev('/');
		if (index != -1)
			tag = tag.mid(index + 1);

		index = tag.findRev('.');
		if (index != -1)
			tag.truncate(index);

		KSimpleConfig entry(*it);
		entry.setGroup(QString::fromLatin1("KCM Locale"));
		QString name = entry.readEntry(QString::fromLatin1("Name"), i18n("without name"));

		combo->insertSubmenu( name, '-' + tag, sub );
	}

	// add all languages to the list
	QStringList countrylist = KGlobal::dirs()->findAllResources("locale", sub + "*/entry.desktop");
	countrylist.sort();

	for ( QStringList::ConstIterator it = countrylist.begin(); it != countrylist.end(); ++it ) {
		KSimpleConfig entry(*it);
		entry.setGroup(QString::fromLatin1("KCM Locale"));
		QString name = entry.readEntry(QString::fromLatin1("Name"), i18n("without name"));
		QString submenu = '-' + entry.readEntry("Region");

		QString tag = *it;
		int index = tag.findRev('/');
		tag.truncate(index);
		index = tag.findRev('/');
		tag = tag.mid(index+1);
		int menu_index = combo->containsTag(tag) ? -1 : -2;
		QPixmap flag( locate( "locale", QString::fromLatin1("l10n/%1/flag.png").arg(tag) ) );
		QIconSet icon( flag );
		combo->insertItem( icon, name, tag, submenu, menu_index );
	}
}

void KCountryPage::fillLanguageMenu(KLanguageButton *combo) {
	combo->clear();
	int menu_index = -2;
	QString submenu; // we are working on this menu
	QStringList langlist = flang->getLangList();
	QMap<QString,QString> langmap = flang->getLangMap();
	QStringList::ConstIterator it;
	for ( it = langlist.begin(); it != langlist.end(); ++it ) {
		if ((*it).isNull()) {
			combo->insertSeparator();
			submenu = QString::fromLatin1("other");
			combo->insertSubmenu(i18n("Other"), submenu, QString::null, -2);
			menu_index = -1; // first entries should _not_ be sorted
			continue;
		}
		combo->insertItem(langmap[(*it)], (*it), submenu, menu_index);
	}
}

/** No descriptions */
bool KCountryPage::save(KLanguageButton *comboCountry, KLanguageButton *comboLang) {
	kdDebug() << "KCountryPage::save()" << endl;
	KConfigBase *config = KGlobal::config();

	config->setGroup(QString::fromLatin1("Locale"));
	config->writeEntry(QString::fromLatin1("Country"), comboCountry->currentTag(), true, true);
	config->writeEntry(QString::fromLatin1("Language"), comboLang->currentTag(), true, true);
	config->sync();

	// only make the system reload the language, if the selected one deferes from the old saved one.
	if (b_savedLanguageChanged) {
		// Tell kdesktop about the new language
		QCString replyType; QByteArray replyData;
		QByteArray data, da;
		QDataStream stream( data, IO_WriteOnly );
		stream << comboLang->currentTag();
		if ( !kapp->dcopClient()->isAttached() )
			kapp->dcopClient()->attach();
		// ksycoca needs to be rebuilt
		KProcess proc;
		proc << QString::fromLatin1("kbuildsycoca");
		proc.start(KProcess::DontCare);
		kdDebug() << "KLocaleConfig::save : sending signal to kdesktop" << endl;
		// inform kicker and kdeskop about the new language
		kapp->dcopClient()->send( "kicker", "Panel", "restart()", QString::null);
		// call, not send, so that we know it's done before coming back
		// (we both access kdeglobals...)
		kapp->dcopClient()->call( "kdesktop", "KDesktopIface", "languageChanged(QString)", data, replyType, replyData );
	}
	// KPersonalizer::next() probably waits for a return-value
	return true;
}

void KCountryPage::setLangForCountry(int i) {
	QString country = cb_country->tag(i);

	KSimpleConfig ent(locate("locale", "l10n/" + country + "/entry.desktop"), true);
	ent.setGroup(QString::fromLatin1("KCM Locale"));
	langs = ent.readListEntry(QString::fromLatin1("Languages"));

	QString lang = QString::fromLatin1("en_US");
	// use the first INSTALLED langauge in the list, or default to C
	for ( QStringList::Iterator it = langs.begin(); it != langs.end(); ++it ) {
		if (cb_language->containsTag(*it)) {
			lang = *it;
			break;
		}
    }

	cb_language->setCurrentItem(lang);
	setLanguageChanged();
}

void KCountryPage::setLanguageChanged() {
	// is the selcted language the same like the one in kdeglobals from before the start?
	b_savedLanguageChanged = (flang->getOldLang() != cb_language->currentTag().lower());
	// is the selected language the same like the one we started kp with from main.cpp?
	b_startedLanguageChanged = (s_oldlocale != cb_language->currentTag());
}


#include "kcountrypage.moc"
