#ifndef __DESKTOPFILE_H__
#define __DESKTOPFILE_H__ 1
#include <kdesktopfile.h>
#include <list>

/** @brief desktop file parser with support for VFolder extensions
 * @author Bernhard Rosenkraenzer <bero@redhat.com>
 */
class DesktopFile:public KDesktopFile {
public:
	DesktopFile(QString const &fileName, bool readOnly=false, const char *resType="apps");
	QString	onlyShowIn() const;
	QStringList readCategories() const;
};

/** @brief handler for multiple desktop files
 * @author Bernhard Rosenkraenzer <bero@redhat.com>
 */
class DesktopFiles:public std::list<DesktopFile*> {
public:
	/** @brief Constructor - Create empty list of desktop files. */
	DesktopFiles();
	/** @brief Constructor - Create list of desktop files
	 * @param dir Directory containing the desktop files
	 * @param subdirs If @p true, subdirectories will be included
	 * @param sdirs If not 0, a list of subdirectories is appended to @p dirs
	 */
	DesktopFiles(QString const &dir, bool subdirs=false, QStringList *sdirs=0);
	/** @brief Destructor - free up space */
	~DesktopFiles();
	/** @brief Add desktop files located in a directory
	 * @param dir Directory containing the desktop files
	 * @param subdirs If @p true, subdirectories will be included
	 * @param sdirs If not 0, a list of subdirectories is appended to @p dirs
	 */
	void addDirectory(QString const &dir, bool subdirs=false, QStringList *sdirs=0);
	/** @brief Add desktop files located in directories
	 * @param dir Colon-separated list of directories containing the desktop files
	 * @param subdirs If @p true, subdirectories will be included
	 * @param sdirs If not 0, a list of subdirectories is appended to @p dirs
	 */
	void addDirectories(QString const &dirs, bool subdirs=false, QStringList *sdirs=0);
private:
	QStringList dirs;
};
#endif
