#ifndef __VFOLDER_H__
#define __VFOLDER_H__ 1
#include <list>
#include <qstringlist.h>
#include "desktopfile.h"

namespace VFolder {
	class VFolderEntry;
	class VFolderQuery;

	enum Type { File, Dir };
	enum QueryType { And, Or };

	typedef std::list<VFolderEntry*> VFolderEntries;

	class VFolderQuery {
	public:
		VFolderQuery(QStringList const &query=QString::null, QueryType type=And, bool reverse=false);
		void		addKeyword(QString const &key) { _query.append(key); }
		QStringList const &query() const { return _query; }
		QueryType	type() const { return _type; }
		bool		reverse() const { return _not; }
		bool		match(DesktopFile const &d) const;
		/// useful for debugging
		operator	QString() const;
	protected:
		QStringList	_query;
		QueryType	_type;
		bool		_not;
	};

	class VFolderQueries:public std::list<VFolderQuery*> {
	public:
		VFolderQueries();
		~VFolderQueries();
		void		append(VFolderQuery *q) { insert(end(), q); }
		bool		match(DesktopFile const &d) const;
	};

	class VFolderEntry {
	public:
		VFolderEntry(Type t=Dir, VFolderEntry *p=0);
		VFolderEntry	*parent() const { return _parent; }
		void		setName(QString const &name) { _name=name; }
		QString const &	name() const { return _name; }
		void		setDesktop(QString const &desktop) { _desktop=desktop; }
		QString const &	desktop() const { return _desktop; }
		void		append(VFolderEntry *entry) { _children.insert(_children.end(), entry); }
		void		addQuery(VFolderQuery *e);
		void		addQueries(VFolderQueries *e);
		VFolderEntries &children() { return _children; }
		VFolderQueries &query() { return _query; }
		Type		type() const { return _type; }
		bool		match(DesktopFile const &d) const { return _query.match(d); }

		void            setDontShowIfEmpty(bool dsie) { _dontshowifempty = dsie; }
		bool            dontShowIfEmpty() { return _dontshowifempty; }

	private:
		VFolderEntry	*_parent;
		Type		_type;
		VFolderEntries	_children;
		VFolderQueries	_query;
		QString		_name;
		QString		_desktop;
		bool		_dontshowifempty;
		bool		_onlyunallocated;
	};

}
#endif
